##################### PSQF 6243 Example 4a using R ##############################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=140, digits=5, scipen=4, show.signif.stars=FALSE)

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import excel XLSX data as table

#if (!require("psych")) install.packages("psych")
#library(psych) # To add descriptive summary functions

if (!require("supernova")) install.packages("supernova")
library(supernova) # To get sums of squares table

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("lmhelprs")) install.packages("lmhelprs")
library(lmhelprs) # To get F-test and change in R2 in GLMs

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files using code below

# Clear environment (run as needed)
#rm(list=ls())

################################################################################
#####              BEGIN FUNCTIONS TO AUTOMATE CALCULATIONS                #####
#####       JUST RUN THIS CODE AS IS; SHOULD NOT NEED TO CHANGE IT         #####
################################################################################

# This SummaryCI function saves a summary of a model object, and then adds 
#  confidence intervals (default is 95%) to the fixed effects table 
SummaryCI = function(model, level=.95){
  modelsummary = summary(model)        # Save model summary
  CIs = confint(model, level=level)    # Save fixed effects CIs
  result = data.frame(modelsummary$coefficients, CIs) # Combine into one table and rename
  colnames(result) = c("Estimate","StdErr","t-value","p-value","LowerCI","UpperCI")
  return(result) # Print result
}

# This FixedEffectSizes function saves a summary of an lm model object, and then adds
#   columns to the summary for Partial D (Cohen's D), Partial R, Semi-Partial R, 
#   Partial R2, and Semi-Partial R2 for all fixed effects
FixedEffectSizes = function(model){
  fullModelSummary = summary(model)  # Save model summary and compute effect sizes
  pD = (2*fullModelSummary$coefficients[,"t value"])/sqrt(model$df.residual)  # Cohen's d
  pR =    fullModelSummary$coefficients[,"t value"]/
     sqrt(fullModelSummary$coefficients[,"t value"]^2+model$df.residual)      # Partial R
  sR =    fullModelSummary$coefficients[,"t value"]*
     sqrt((1-summary(model)$r.squared)/model$df.residual)                     # Semi-Partial R
  result = data.frame(fullModelSummary$coefficients, pD, pR, sR, pR^2, sR^2)  # Merge to one table
  result = subset(result, select=-c(Std..Error,t.value)) # Drop unneeded columns
  colnames(result) = c("Estimate","p-value","Partial-d","Partial-r",  # Rename columns
                       "SemiPartial-r","Partial-R2","SemiPartial-R2")
  return(result) # Print result
}

# This glhtSummaryCI function saves a summary of a glht object, and then adds 
#  confidence intervals (default is 95%) to the summary for the linear combinations 
glhtSummaryCI = function(glhtObject, level=.95){
  glhtSummary = summary(glhtObject, test=adjusted("none")) # Save unadjusted p-values
  CIs = confint(glhtObject, level=level, calpha=univariate_calpha()) # Get CIs
  result = data.frame(Estimate=glhtSummary$test$coefficients, 
                      StdErr=glhtSummary$test$sigma,
                      pvalue=glhtSummary$test$pvalues, CIs$confint) # Merge to one table
  result = subset(result, select=-c(Estimate.1))
  colnames(result) = c("Estimate","StdErr","p-value","LowerCI","UpperCI")
  return(result) # Print result
}

# This glhtEffectSizes function saves a summary of a glht object, and then adds
#   columns to that summary for Partial D (Cohen's D), Partial R, Semi-Partial R, 
#   Partial R2, and Semi-Partial R2 for all linear combinations
glhtEffectSizes = function(glhtObject, modelObject, level=.95){
  glhtSummary = summary(glhtObject, test=adjusted("none")) # save unadjusted p-values
  pD = (2*glhtSummary$test$tstat)/sqrt(modelObject$df.residual)
  pR = glhtSummary$test$tstat/sqrt(glhtSummary$test$tstat^2+modelObject$df.residual)
  sR = glhtSummary$test$tstat*sqrt((1-summary(modelObject)$r.squared)/modelObject$df.residual)
  result = data.frame(Estimate=glhtSummary$test$coefficients, pvalue=glhtSummary$test$pvalues, 
                      pD, pR, sR, pR^2, sR^2)
  colnames(result) = c("Estimate","p-value","Partial-d","Partial-r",  # Rename columns
                       "SemiPartial-r","Partial-R2","SemiPartial-R2")
  return(result) # Print result
}

# This R2changeF function prints the F-test and change in R2 for a model comparison
#   (as provided by the hierarchical_lm function in the lmhelprs package),
#   as well as partial R2 and semi-partial R2 (which is equal to R2 change here)
R2changeF = function(ReducedModel, FullModel, PredName){
  FullModelTable = supernova(FullModel) # Get total sums of squares for computations
  SStotal = FullModelTable[["tbl"]][["SS"]][length(FullModelTable[["tbl"]][["SS"]])]  
  R2F = hierarchical_lm(ReducedModel, FullModel) # Get values for F-test and R2 change
  SSeffect = R2F[2,7]    # Save effect sums of squares (as reduction in SS from full model)
  SSresidual = R2F[2,5]  # Save residual sums of squares from full model
  pr2 = SSeffect/(SSeffect+SSresidual)  # Compute Partial R2
  sr2 = SSeffect/SStotal                # Compute Semi-Partial R2
  cat(paste("F-Test and R2 Change for", PredName, "Slopes \n")) # Add better title
  colnames(R2F) = c("AdjR2","R2","R2change","DFden","RSS", "DFnum","SSeffect",
                    "Fvalue","pvalue")  # Better names
  result = data.frame(R2F$R2, R2F$R2change, R2F$DFnum, R2F$DFden, R2F$Fvalue, R2F$pvalue, 
                      pr2, sr2) # Re-order, add effect sizes computed above
  colnames(result) = c("R2-total","R2-change","DF-num","DF-den","F-value", 
                       "p-value","Partial-R2","SemiPartial-R2") # Rename for easier reading
  return(result[2,]) # Only print second row (first is no longer needed)
}


################################################################################
#####            BEGIN EXAMPLE 4a DATA IMPORT AND MANIPULATION             #####
################################################################################

# Set working directory (to import and export files to)
# Paste in the folder address where "GSS_Example.xlsx" is saved in quotes
setwd("C:/Dropbox/24_PSQF6243/PSQF6243_Example4a")

# Import GSS_Example.xlsx data from working directory -- path = file name
Example4a = read_excel(path="GSS_Example.xlsx", sheet="GSS_Example") 
# Convert to data frame to use for analysis
Example4a = as.data.frame(Example4a)

### Create and label predictor variables for model 1 (same as in Example 2)
# Linear predictor for education centered so that 0 is meaningful
Example4a$educ12=Example4a$educ-12 
# educ12: Education (0=12 years)
# Recode marry predictor so that 0 is meaningful
Example4a$marry01=NA  # Create new empty variable, then recode 
Example4a$marry01[which(Example4a$marry==1)]=0 
Example4a$marry01[which(Example4a$marry==2)]=1
# marry01: 0=unmarried, 1=married 

### Create and label predictor variables for model 2 (same as in Example 3)
# 2 Indicator-dummy-coded binary predictors for workclass
Example4a$LvM=NA; Example4a$LvU=NA  # Make 2 new empty variables 
Example4a$LvM[which(Example4a$workclass==1)]=0  # Replace each for lower
Example4a$LvU[which(Example4a$workclass==1)]=0
Example4a$LvM[which(Example4a$workclass==2)]=1  # Replace each for middle
Example4a$LvU[which(Example4a$workclass==2)]=0
Example4a$LvM[which(Example4a$workclass==3)]=0  # Replace each for upper
Example4a$LvU[which(Example4a$workclass==3)]=1
# LvM: Low=0 vs Mid=1 Class
# LvU: Low=0 vs Upp=1 Class

# Center age at 18 (minimum in sample)
Example4a$age18=Example4a$age-18 
# age18: Age (0=18 years) 

# 3 Piecewise slopes for education
# Make 3 new empty variables 
Example4a$lessHS=NA; Example4a$gradHS=NA; Example4a$overHS=NA 
# Replace each for educ less than 12
Example4a$lessHS[which(Example4a$educ<12)]=Example4a$educ[which(Example4a$educ<12)]-11  
Example4a$gradHS[which(Example4a$educ<12)]=0
Example4a$overHS[which(Example4a$educ<12)]=0  
# Replace each for educ greater or equal to 12
Example4a$lessHS[which(Example4a$educ>=12)]=0 
Example4a$gradHS[which(Example4a$educ>=12)]=1  
Example4a$overHS[which(Example4a$educ>=12)]=Example4a$educ[which(Example4a$educ>=12)]-12
# lessHS: Slope for Years Ed Less Than High School
# gradHS: Acute Bump for Graduating High School
# overHS: Slope for Years Ed After High School
# Label outcome variable
# income: Annual Income in 1000s

# Open external file to save results to
txtStart(file="Example4a_R_Output.txt")


################################################################################
#####       MODEL 1: DEMONSTRATING SINGLE-SLOPE PREDICTORS COMBINED        #####
################################################################################

print("R Model 1: Linear Education and Binary Marital Status")
Model1 = lm(data=Example4a, formula=income~1+educ12+marry01)
supernova(Model1)             # supernova prints sums of squares and residual variance
SummaryCI(Model1, level=.95)  # custom function to add CIs to fixed effects table
FixedEffectSizes(Model1)      # custom function to add effect sizes for fixed slopes

txtComment(" ") # insert blank space in output  


################################################################################
#####       MODEL 2: DEMONSTRATING MULTIPLE-SLOPE PREDICTORS COMBINED      #####
################################################################################

print("R Model 2: Three-Category Workclass, Quadratic Age, and Piecewise Education")
Model2 = lm(data=Example4a, formula=income~1+LvM+LvU+age18+I(age18^2)+lessHS+gradHS+overHS)
supernova(Model2)   # supernova prints sums of squares and residual variance
SummaryCI(Model2, level=.95)  # custom function to add CIs to fixed effects table
FixedEffectSizes(Model2)      # custom function to add effect sizes for fixed slopes

txtComment(" ") # insert blank space in output  

print("R Ask for missing model-implied group difference")
glhtModel2 = glht(model=Model2, linfct=rbind("Mid vs Upp Diff" = c(0,-1,1,0,0,0,0,0)))
glhtSummaryCI(glhtModel2, level=.95)  # custom function to add CIs to glht output table
glhtEffectSizes(glhtObject=glhtModel2, modelObject=Model2, 
                level=.95) # custom function to compute glht effect sizes 

txtComment(" ") # insert blank space in output  

### How to get F-test and change in R2 for multiple fixed slopes at once ###

# Fit model without fixed slopes of interest (LvM and LvU for workclass here)
Model2NoClass = lm(data=Example4a, formula=income~1+age18+I(age18^2)+lessHS+gradHS+overHS)
# Get F-test and effect sizes for fixed slopes of interest using custom function
R2changeF(ReducedModel=Model2NoClass, FullModel=Model2, PredName="Workclass")

txtComment(" ") # insert blank space in output  

# Repeat for age slopes 
Model2NoAge = lm(data=Example4a, formula=income~1+LvM+LvU+lessHS+gradHS+overHS)
R2changeF(ReducedModel=Model2NoAge, FullModel=Model2, PredName="Age")

txtComment(" ") # insert blank space in output  

# Repeat for education slopes
Model2NoEduc = lm(data=Example4a, formula=income~1+LvM+LvU+age18+I(age18^2))
R2changeF(ReducedModel=Model2NoEduc, FullModel=Model2, PredName="Education")

txtComment(" ") # insert blank space in output  


#################################################################################
# Close output text file
txtStop()


