// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

********************************************************************************
******           BEGIN DATA MANIPULATION OF EXAMPLE 4A                   *******
******               CHANGE cd to your directory                         *******
********************************************************************************

// Paste in the folder address where "GSS_Example.xlsx" is saved between " "
cd "C:\Dropbox\24_PSQF6243\PSQF6243_Example4a"
// Using UIowa virtual desktop instead
//cd "\\Client\C:\Dropbox\24_PSQF6243\PSQF6243_Example4a"

// IMPORT GSS_Example.xlsx data from working directory using exact file name
// To change all variable names to lowercase, remove "case(preserve")
clear // Clear before means close any open data
import excel "GSS_Example.xlsx", case(preserve) firstrow clear 
// Clear after means re-import if it already exists (if need to start over)

// Create and label predictor variables for model 1 (same as in Example 2)
// Linear predictor for educaiton centered so that 0 is meaningful
gen educ12=educ-12
label variable educ12 "educ12: Education (0=12 years)"
// Recode marry predictor so that 0 is meaningful
gen marry01=. // Create new empty variable, then recode
replace marry01=0 if marry==1
replace marry01=1 if marry==2
label variable marry01 "marry01: 0=unmarried, 1=married"

// Create and label predictor variables for model 2 (same as in Example 3)
// 2 Indicator-dummy-coded binary predictors for workclass
gen LvM=. // Make two new empty variables
gen LvU=.
replace LvM=0 if workclass==1 // Replace each for lower
replace LvU=0 if workclass==1
replace LvM=1 if workclass==2 // Replace each for middle
replace LvU=0 if workclass==2
replace LvM=0 if workclass==3 // Replace each for upper
replace LvU=1 if workclass==3
label variable LvM "LvM: Low=0 vs Mid=1 Class"
label variable LvU "LvU: Low=0 vs Upp=1 Class"

// Center age at 18 (minimum in sample)
gen age18 = age-18
label variable age18 "age18: Age (0=18 years)"

// 3 Piecewise slopes for education
gen lessHS=. // Make 3 new empty variables
gen gradHS=.
gen overHS=.
// Replace for educ less than 12
replace lessHS=educ-11 if educ <  12
replace gradHS=0       if educ <  12
replace overHS=0       if educ <  12
// Replace for educ greater or equal to 12
replace lessHS=0       if educ >= 12 
replace gradHS=1       if educ >= 12
replace overHS=educ-12 if educ >= 12
// Label variables
label variable lessHS "lessHS: Slope for Years Ed Less Than High School"
label variable gradHS "gradHS: Acute Bump for Graduating High School"
label variable overHS "overHS: Slope for Years Ed After High School"
// Label outcome
label variable income "income: Annual Income in 1000s"

// Open external file to save results to
log using Example4a_STATA_Output.log, replace 


********************************************************************************
*******    MODEL 1: DEMONSTRATING SINGLE-SLOPE PREDICTORS COMBINED       *******
********************************************************************************

display "STATA Model 1: Linear Education and Binary Marital Status"
regress income c.educ12 c.marry01, level(95)
matrix Model1 = r(table)  // Save results for computing effect sizes
matrix list Model1        // Show saved results    d = 2t/SQRT(DFden)
display "Partial Cohen's d for marry01 = " 2*Model1[3,2]/sqrt(Model1[7,2])

display "STATA Semi-Partial and Partial r and R2 Effect Sizes for Fixed Slopes"
pcorr income c.educ12 c.marry01


********************************************************************************
*******    MODEL 2: DEMONSTRATING MULTIPLE-SLOPE PREDICTORS COMBINED     *******
********************************************************************************

display "STATA Model 2: Three-Category Workclass, Quadratic Age, and Piecewise Education"
regress income c.LvM c.LvU c.age18 c.age18#c.age18 c.lessHS c.gradHS c.overHS, level(95)
estimates store Model2      // Save all model results
ereturn list                // See what has been stored automatically 
global SSresidual = e(rss)  // Save full model SS residual for effect sizes below
global SSfull = e(mss)      // Save full model SS model for effect sizes below

display "STATA Partial and Semi-Partial R and R2 Effect Sizes for Fixed Slopes"
pcorr income c.LvM c.LvU c.age18 c.age18#c.age18 c.lessHS c.gradHS c.overHS

display "STATA Get Model-Implied Difference for Middle vs Upper and Effect Sizes"
estimates restore Model2    // Restore model results for post-estimations below 
lincom c.LvM*-1 + c.LvU*1   // Code below should work after any lincom
   display "Partial D= "      (2*(r(estimate)/r(se)))/sqrt(r(df))
   display "Partial R= "      (r(estimate)/r(se))/sqrt((r(estimate)/r(se))^2+r(df))
   display "Semi-Partial R= " (r(estimate)/r(se))*sqrt((1-e(r2))/r(df))
   
/// How to get F-test and change in R2 for multiple fixed slopes at once ///

display "STATA F-test for Overall Effect of Workclass"
test (c.LvM=0)(c.LvU=0)

display "STATA F-test for Overall Effect of Age"    
test (c.age18=0)(c.age18#c.age18=0)  

display "STATA F-test for Overall Effect of Education"
test (c.lessHS=0)(c.gradHS=0)(c.overHS=0) 

display "STATA Reduced Model to Get Model SS from Omitting Workclass"
quietly regress income c.age18 c.age18#c.age18 c.lessHS c.gradHS c.overHS, level(95)
global  SSeffect = $SSfull - e(mss)
display "Partial R2 = "      $SSeffect/($SSeffect+$SSresidual)
display "Semi-Partial R2 = " $SSeffect/($SSfull+$SSresidual)

display "STATA Reduced Model to Get Model SS from Omitting Age"
quietly regress income c.LvM c.LvU c.lessHS c.gradHS c.overHS, level(95)
global  SSeffect = $SSfull - e(mss)
display "Partial R2 = "      $SSeffect/($SSeffect+$SSresidual)
display "Semi-Partial R2 = " $SSeffect/($SSfull+$SSresidual)

display "STATA Reduced Model to Get Model SS from Omitting Education"
quietly regress income c.LvM c.LvU c.age18 c.age18#c.age18, level(95)
global  SSeffect = $SSfull - e(mss)
display "Partial R2 = "      $SSeffect/($SSeffect+$SSresidual)
display "Semi-Partial R2 = " $SSeffect/($SSfull+$SSresidual)

display "Extra STATA code for how to use nestreg to get F-test for R2 change"
gen age18sq=age18*age18  // Because nestreg won't allow interactions on the fly
// Format below is outcome (Full model)(what to remove to test those slopes)
// Here, we remove LvM and LvU to test overall effect of workclass
nestreg: regress income (age18 age18sq lessHS gradHS overHS)(LvM LvU)


// Remove all macros (so they don't get used in other STATA programs)
macro drop _all    
      
// Close external file of saved results
log close  

   
   



